function [bestConfig,bestTransMat,sampledError] = FastMatch(template,img,... % mandatory
    epsilon,delta,photometricInvariance,minScale,maxScale) %optional                                                                        
%FastMatch Find an approximate template match under affine transformations.
%   FastMatch(template,img,... % mandatory
%    [epsilon=0.15],[delta=0.25],[photometricInvariance=false],[minScale=0.5],[maxScale=2]) %optional   
%  The template and img inputs should be uint8 grayscale images

%% set default values for optional variables
if (~exist('epsilon','var'))
    epsilon = 0.15;
end
if (~exist('delta','var'))
    delta = 0.25;
end
if (~exist('photometricInvariance','var'))
    photometricInvariance = 0;
end
if (~exist('minScale','var'))
    minScale = 0.5;
end
if (~exist('maxScale','var'))
    maxScale = 2;
end


%% formatting the image and template
img = MakeOdd(img);
template = MakeOdd(template);

if ( ~strcmp(class(img),'double') || ~strcmp(class(template),'double')) %#ok<STISA>
    error('FastMatch: img and template should both be of class ''double'' (in the range [0,1])');
end

if ( (~size(img,3)==1) || (~size(template,3)==1) )
    error('img and template should both be grayscale images');
end



%% image dimensions
[h1,w1] = size(template);
[h2,w2] = size(img);
r1x = 0.5*(w1-1);
r1y = 0.5*(h1-1);
r2x = 0.5*(w2-1);
r2y = 0.5*(h2-1);


%% search range
minTx = -(r2x-r1x*minScale);
maxTx = r2x-r1x*minScale;
minTy = -(r2y-r1y*minScale);
maxTy = r2y-r1y*minScale;
minR = -pi;
maxR = pi;



% parametrize the initial grid
[bounds,steps] = GenerateGrid(w1,h1,delta,minTx,maxTx,minTy,maxTy,minR,maxR,minScale,maxScale);


%% the actual search!

[bestConfig,bestTransMat,sampledError] = ...
    FindBestTransformation(template,img,bounds,steps,epsilon,delta,photometricInvariance);
return

