function FastMatch_demo
%%%%%%%%%%%%%%%%%%%%%%%

clc
% clear all 
close all
dbstop if error


% adding 2 subdirectories to Matlab PATH
AddPaths

% compiling the relevant Mex-files
CompileMex


%% Example 1: Locating a predefined template in the image
disp('Example 1: Locating a predefined template in an image (thai food)');
fprintf('======================================================\n');

fprintf('==> PRESS ANY KEY TO CONTINUE\n\n');

pause

% reading image and template
img =       imread([pwd '/example/image.png']);
img =       im2double(img);
template =  imread([pwd './example/template.png']);
template =  im2double(template);

% reading the 'ground-truth' transformation (not mandatory)
load([pwd '/example/OptMat.mat'], 'optMat');

[bestConfig,bestTransMat,sampledError] =  FastMatch(template,img);

[optError,fullError,overlapError] = MatchingResult(template,img,bestTransMat, optMat,'Example 1'); %#ok<NODEF>
fprintf('example 1 -  optError: %.3f, fullError: %.3f, overlapError: %.3f\n\n',optError,fullError,overlapError);

fprintf('Example 1: finished\n\n');

disp('--------------------------------------');
disp('Loading an image for examples 2 and 3...');
disp('(In experiment 2 - a random template will be chosen from it)');
fprintf('(In experiment 3 - choose the template manually by defining 3 corners of a parallelogram)\n\n');

fprintf('==> PRESS ANY KEY AND THEN CHOOSE THE IMAGE FILE FROM THE SUGGESTED DIRECTORY\n\n');

pause 

%% Load initial image for examples 2 and 3
[imgFileName, PathName, ~] = uigetfile('*',...
               'Choose an image for examples 2 and 3',...
               '.\imgs\keyboard.jpg');
FullName = [PathName,imgFileName];
img = imread(FullName);
[~,~,d2] = size(img);
if (d2>1)
    img = rgb2gray(img);
end
img = im2double(img);


%% example 2: locating a randomly generated template in the image
disp('Example 2: locating a randomly generated template in the image');
fprintf('======================================================\n');

fprintf('==> PRESS ANY KEY TO CONTINUE\n\n');

pause

[template,optMat] = GenerateRandomAffineTemplate(img);

[bestConfig,bestTransMat,sampledError] = FastMatch(template,img);

[optError,fullError,overlapError] = MatchingResult(template,img,bestTransMat,optMat,'Example 2');
fprintf('example 2 -  optError: %.3f, fullError: %.3f, overlapError: %.3f',optError,fullError,overlapError);

fprintf('Example 2: finished\n\n');



%% example 3: locating a user selected template in the image
disp('Example 3: locating a user selected template in the image');
fprintf('======================================================\n');

fprintf('==> PRESS ANY KEY TO CONTINUE to example 3\n');
fprintf('You will need to mark 3 corners of a parallelogram using a triangle\n');
fprintf('(don''t make smaller than original)\n\n');
fprintf('(move/stretch/rotate the suggested triangle and DOUBLE-CLICK it)\n\n');

pause 

[template,optMat] = GenerateUserSelectedTemplate(img,'Example ');

[bestConfig,bestTransMat,sampledError] = FastMatch(template,img);

[optError,fullError,overlapError] = MatchingResult(template,img,bestTransMat,optMat,'Example 3');
fprintf('example 3 -  optError: %.3f, fullError: %.3f, overlapError: %.3f',optError,fullError,overlapError);

fprintf('Example 3: finished\n\n');

disp('--------------------------------------');

%% example 4: locating a user selected template from one image in another related image
disp('Example 4: locating a user selected template from one image in another related image');
fprintf('======================================================\n');

disp('Loading a target image and a template image for example 4...');
fprintf('==> PRESS ANY KEY AND THEN CHOOSE first the target image and then the template image\n');
fprintf('(make sure to choose target and template with same index (1-5))\n\n');
fprintf('(then - move or enlarge the suggested template and DOUBLE-CLICK it)\n\n');

pause 

[imgFileName,PathName,~] = uigetfile('*',...
            'Choose an image to match with the template',...
            '.\imgPairs\image__*.*');
FullName = [PathName,imgFileName];

img = imread(FullName);

[~,~,d2] = size(img);
if (d2>1)
    img = rgb2gray(img);
end
img = im2double(img);

[template] = GenerateUserSelectedOtherImage(img);

[bestConfig,bestTransMat,sampledError] = FastMatch(template,img);

[optError,fullError,overlapError] = MatchingResult(template,img,bestTransMat,[],'Example 4'); % Note that there's
% no "Ground truth"
fprintf('example 4 -  optError: %.3f, fullError: %.3f, overlapError: %.3f',optError,fullError,overlapError);

fprintf('Example 4: finished\n\n');

fprintf('END OF DEMO!\n\n');

fprintf('==> PRESS ANY KEY TO END\n');

pause

keyboard

return
end


function [template,optMat] = GenerateRandomAffineTemplate(img)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% pick a random transformation and continue if it is in bounds

[h,w] = size(img);

minDim2 = min(h,w); % minimal dimension

sizeFact = 3; % relative size of template dimension w.r.t. image dimension

n1 = ceil(minDim2/sizeFact);

if (mod(n1,2)==0)
    n1 = n1 - 1;
end

r1 = 0.5*(n1-1);
rx = 0.5*(w-1);
ry = 0.5*(h-1);

%% affine search limits

c = 2;

minTx = -(rx-r1/c);
maxTx = rx-r1/c;
minTy = -(ry-r1/c);
maxTy = ry-r1/c;
minR = -pi;
maxR = pi;
minS = 1/c;
maxS = c;

rangeTx = maxTx - minTx;
rangeTy = maxTy - minTy;
rangeR = maxR - minR;
rangeS = maxS - minS;

attempt = 0;
while(1)
    attempt = attempt + 1;
    randVec = rand(1,6);
    randAffVec = [minTx,minTy,minR,minS,minS,minR] + randVec.*[rangeTx,rangeTy,rangeR/4,rangeS,rangeS,rangeR];
    %             randAffVec = [0 0 0 1 1 0];
    randAffMat = CreateAffineTransformation(randAffVec);
    %         randAff = maketform('affine',randAffMat');
    cornersX = [1 n1 n1 1];
    cornersY = [1 1 n1 n1];
    randAff2x2 = randAffMat(1:2,1:2);
    cornersTest = round(randAff2x2*[cornersX-(r1+1);cornersY-(r1+1)]);
    cornersTestxs = round(cornersTest(1,:) + (rx+1)  + randAffMat(1,3));
    cornersTestys = round(cornersTest(2,:) + (ry+1)  + randAffMat(2,3));
    
    % check if mapped into I2 bounds
    if (isequal(BoundBy(cornersTestxs,1,w),cornersTestxs) && (isequal(BoundBy(cornersTestys,1,h),cornersTestys)))
        %% Construct the template I1
        % get the inverse transform
        centeredTL3 = [1,1; n1,1; n1,n1] - (r1+1); % BL,TL,TR
        p = [cornersTestxs' cornersTestys'];
        aff = cp2tform(centeredTL3,p(1:3,:),'affine');
        
        template = imtransform(img,fliptform(aff),'nearest','XData',[1 n1] - (r1+1),'YData',[1 n1] - (r1+1));
        % verify that it isn't just flat
        if (std(double(template(:)))>0.1)
            break;
        end
    end
end

optMat = randAffMat;

return;
end

function [template,optMat] = GenerateUserSelectedTemplate(img,prefixName)

[h,w] = size(img);

minDim2 = min(h,w); % minimal dimension

sizeFact = 3; % relative size of template dimension w.r.t. image dimension

n1 = ceil(minDim2/sizeFact);

n12 = round(minDim2/2);
n34 = n12+n1-1;


r1 = 0.5*(n1-1);
rx = 0.5*(w-1);
ry = 0.5*(h-1);


%% get parallelogram in I2 from the user and transform it to a square (= query image I1)
tempfig = figure; imshow(img); 
title('adjust (dont enlarge) the triangle (half parallelogram) and them DOUBLE CLICK it');
set(gcf,'name',[prefixName ': template and image']);

hand = impoly(gca,[n34,n12; n12,n12; n12,n34]);
p = wait(hand);

p1 = [p(1,1),p(1,2)];
p2 = [p(2,1),p(2,2)];
p3 = [p(3,1),p(3,2)];
mid = 0.5*(p1+p3);
opp = p2 + 2*(mid-p2);



% Sides of the parallelogream
s1 = norm(p1-p2);
s2 = norm(p3-p2);
smallestRelation = min(s1/n1, s2/n1);
largestRelation  = max(s1/n1, s2/n1);
if ((smallestRelation < 0.5) || (largestRelation > 2))
    warning('Template chosen has dimesions too small or too large for demo parameters. You can change the value of the variable "sizeFact" in the FastMAtch_demo file.'); %#ok<WNTAG>
end
hold off;


centeredTL3 = [n1,1; 1,1; 1,n1] - (r1+1); % BL,TL,TR
aff = cp2tform(centeredTL3,p,'affine');

template = imtransform(img,fliptform(aff),'bicubic','XData',[1 n1] - (r1+1),'YData',[1 n1] - (r1+1));

%% extract the parameters of the transformation (for ground-truth visualization)
centeredP = [p(:,1)-(rx+1), p(:,2)-(ry+1)];
aff = cp2tform(centeredTL3,centeredP,'affine');

% aff.tdata.T = [1 0 0; 0 1 0; 0 0 1];
optMat = aff.tdata.T';
centerPoint = [r1+1;r1+1];
tcenterpoint = optMat(1:2,1:2)*centerPoint;
vecx = [-rx,rx];
vecy = [-ry,ry];

I1MappedByOPT = imtransform(template,aff,'bicubic','xdata',tcenterpoint(1)+vecx,'ydata',tcenterpoint(2)+vecy,'size',size(img));

% show the images
close(tempfig);
figure;
set(gcf,'name',[prefixName ': template and image']);
subplot 121; imshow(template); title('template');
subplot 122; imshow(img); title('desired location in target img');

subplot 122; hold on; plot([p1(1),p2(1),p3(1),opp(1),p1(1)],[p1(2),p2(2),p3(2),opp(2),p1(2)],'*-g');

return;
end

function [template] = GenerateUserSelectedOtherImage(I2)

%% Get user selected images and format them
[imgFileName,PathName,~] = uigetfile('*',...
            'Choose an image to match with the template',...
            '.\imgPairs\image_for_template__*.*');
FullName = [PathName,imgFileName];

        %% Load initial image for all examples - we'll match different templates for it.
I1 = imread(FullName);

I1 = MakeOdd(I1);

if (max2(I1)==1)
    I1 = uint8(255*double(I1));
end

if (max2(I2)==1)
    I2 = uint8(255*double(I2));
end

[~,~,d1] = size(I1);
if (d1>1)
    I1 = rgb2gray(I1);
end
I1 = im2double(I1);


[~,~,d2] = size(I2);
if (d2>1)
    I2 = rgb2gray(I2);
end
I2 = im2double(I2);


%% display images
[h1,w1] = size(I1);
imgPairHand = figure;
subplot 121; axis on;   imshow(I1);
subplot 122; axis on;   imshow(I2);



subplot 121; hold on;
h = imrect(gca,[w1/2 h1/2 w1/6 w1/6]);
p = wait(h); % p(4) = p(3);
tempXmin = p(1);
tempYmin = p(2);
tempW = p(3);
tempH = p(4);

rectXs = [tempXmin,tempXmin+tempW-1,tempXmin+tempW-1,tempXmin,tempXmin];
rectYs = [tempYmin,tempYmin,tempYmin+tempH-1,tempYmin+tempH-1,tempYmin];

%% define template
template = I1(tempYmin:tempYmin+tempH-1,tempXmin:tempXmin+tempW-1);

return;
end